"use strict";
const metadata_1 = require("../../../metadata");
require("reflect-metadata");
function createRouteDecorator(pathOrMeta, method) {
    if (typeof pathOrMeta === 'string')
        pathOrMeta = { path: pathOrMeta };
    if (typeof method !== 'undefined') {
        if (pathOrMeta.method && pathOrMeta.method != method)
            throw new Error(`Redeclaration of route method: ${pathOrMeta}`);
        pathOrMeta.method = method;
    }
    else
        pathOrMeta.method = pathOrMeta.method || 'get';
    let meta = pathOrMeta;
    return function (controller, routeName, routeFn) {
        var controllerRoutes = Reflect.getOwnMetadata(metadata_1.ControllerRoutesSym, controller) || [];
        if (!controllerRoutes.find(route => route == routeName))
            controllerRoutes.push(routeName);
        Reflect.defineMetadata(metadata_1.ControllerRoutesSym, controllerRoutes, controller);
        var methodRoutes = Reflect.getOwnMetadata(metadata_1.RouteMetadataSym, controller, routeName) || [];
        methodRoutes.push(meta);
        Reflect.defineMetadata(metadata_1.RouteMetadataSym, methodRoutes, controller, routeName);
    };
}
exports.createRouteDecorator = createRouteDecorator;
function Route(meta) {
    return createRouteDecorator(meta);
}
exports.Route = Route;
//# sourceMappingURL=route.decorator.js.map