/// <reference types="sequelize" />
import * as Sql from 'sequelize';
import { StaticModelT, ModelT, PkType, Db, QueryT, CountQueryT, UpdateQueryT, DestroyQueryT } from '../core';
export declare class DbImpl<T extends ModelT<PkType>, TInstance, TAttributes> implements Db<T> {
    private modelFn;
    private model;
    constructor(modelFn: StaticModelT<T>, model: Sql.Model<TInstance, TAttributes>);
    create(t: Object | T | Object[] | T[]): Promise<any>;
    findById(id: string | number): Promise<T | null>;
    findOne(query: QueryT): Promise<T | null>;
    findOrCreate(query: Sql.WhereOptions, defaults?: Object | T): Promise<[T, boolean]>;
    findAndCountAll(query?: QueryT): Promise<{
        count: number;
        results: T[];
    }>;
    findAll(query?: QueryT): Promise<T[]>;
    all(query?: QueryT): Promise<T[]>;
    count(query?: CountQueryT): Promise<number>;
    max(field: string): Promise<any>;
    min(field: string): Promise<any>;
    sum(field: string): Promise<number>;
    save(t: T): Promise<T>;
    update(query: number | string | T | UpdateQueryT, replace: Object): Promise<boolean | number>;
    updateOrCreate(query: Sql.WhereOptions, defaults: Object | T): Promise<[T, boolean]>;
    destroy(query: number | string | T | DestroyQueryT): Promise<any>;
    private isId(query);
    private isT(query);
    private copyVals;
    private createCopyValsFn();
    fromJson(json: any): T;
    private wrapResult(result);
    private wrapResults(results);
}
