"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const metadata_1 = require("../metadata");
const decorators_1 = require("../decorators");
class DbImpl {
    constructor(modelFn, model) {
        this.modelFn = modelFn;
        this.model = model;
        this.copyVals = this.createCopyValsFn();
    }
    create(t) {
        return __awaiter(this, void 0, void 0, function* () {
            if (t instanceof Array) {
                let results = yield this.model.bulkCreate(t);
                return this.wrapResults(results);
            }
            else {
                let result = yield this.model.create(t);
                return this.wrapResult(result);
            }
        });
    }
    findById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.model.findById(id);
            return result && this.wrapResult(result);
        });
    }
    findOne(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.model.findOne(query);
            return result && this.wrapResult(result);
        });
    }
    findOrCreate(query, defaults) {
        return __awaiter(this, void 0, void 0, function* () {
            let [result, created] = yield this.model.findOrCreate({ where: query, defaults: defaults || {} });
            return [result && this.wrapResult(result), created];
        });
    }
    findAndCountAll(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let results = yield this.model.findAndCountAll(query);
            return { count: results.count, results: this.wrapResults(results.rows) };
        });
    }
    findAll(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let results = yield this.model.findAll(query);
            return this.wrapResults(results);
        });
    }
    all(query) {
        return this.findAll(query);
    }
    count(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.count(query);
        });
    }
    max(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.max(field);
        });
    }
    min(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.min(field);
        });
    }
    sum(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.sum(field);
        });
    }
    save(t) {
        return __awaiter(this, void 0, void 0, function* () {
            let [result, created] = yield this.findOrCreate({ id: t.id }, t);
            return result;
        });
    }
    update(query, replace) {
        return __awaiter(this, void 0, void 0, function* () {
            let isId = true;
            if (this.isId(query))
                query = { where: { id: query } };
            else if (this.isT(query))
                query = { where: { id: query.id } };
            else
                isId = false;
            let [affected, results] = yield this.model.update(replace, query);
            return affected;
        });
    }
    updateOrCreate(query, defaults) {
        return __awaiter(this, void 0, void 0, function* () {
            let [result, created] = yield this.findOrCreate(query, defaults);
            if (!created) {
                let worked = yield this.update({ where: query }, defaults);
                if (!worked)
                    throw new Error("Failed to update or create a model.");
                let resultOrNull = yield this.findOne(query);
                if (!resultOrNull)
                    throw new Error("Updated row, but could not find it afterwards.");
                result = resultOrNull;
            }
            return [result, created];
        });
    }
    destroy(query) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isId(query))
                query = { where: { id: query } };
            if (this.isT(query))
                query = { where: { id: query.id } };
            return yield this.model.destroy(query);
        });
    }
    isId(query) {
        return typeof query === 'number' || typeof query == 'string';
    }
    isT(query) {
        return !!query.id;
    }
    createCopyValsFn() {
        let directProps = [];
        let translateProps = [];
        var props = Reflect.getOwnMetadata(metadata_1.ModelPropertiesSym, this.modelFn.prototype) || [];
        for (var q = 0; q < props.length; q++) {
            var propName = props[q];
            var propMeta = Reflect.getOwnMetadata(metadata_1.PropMetadataSym, this.modelFn.prototype, propName);
            if (!propMeta)
                throw new Error(`Could not find model property metadata for property ${this.modelFn.name || this.modelFn}.${propName}.`);
            if (propMeta.type == decorators_1.Types.DATE)
                translateProps.push([propName, dateStr => new Date(dateStr)]);
            else
                directProps.push(propName);
        }
        return function (sql, t) {
            for (var q = 0; q < directProps.length; q++) {
                t[directProps[q]] = sql[directProps[q]];
            }
            for (var q = 0; q < translateProps.length; q++) {
                let propName = translateProps[q][0];
                let translateFn = translateProps[q][1];
                t[propName] = translateFn(sql[propName]);
            }
        };
    }
    fromJson(json) {
        return this.wrapResult(json);
    }
    wrapResult(result) {
        let t = new this.modelFn();
        this.copyVals(result, t);
        return t;
    }
    wrapResults(results) {
        return results.map(result => this.wrapResult(result));
    }
}
exports.DbImpl = DbImpl;
//# sourceMappingURL=db-impl.js.map