"use strict";
const clc_1 = require("../util/clc");
class Logger {
    constructor(logLevel) {
        if (typeof logLevel === 'undefined')
            logLevel = { default: 'warn' };
        if (typeof logLevel === 'string')
            logLevel = { default: logLevel };
        this.logLevel = logLevel;
    }
    logLevelCheck(subsystem, logLevel) {
        let allowedLevel = (subsystem && this.logLevel[subsystem]) || this.logLevel['default'];
        switch (allowedLevel) {
            case 'trace':
                return true;
            case 'verbose':
                return logLevel !== 'trace';
            case 'warn':
                return logLevel !== 'trace' && logLevel !== 'verbose';
            case 'info':
                return logLevel === 'info' || logLevel === 'error';
            case 'error':
                return logLevel === 'error';
        }
    }
    log(subsystem, message, ...optionalParams) {
        if (subsystem)
            console.log(`[${subsystem}]`, message, ...optionalParams);
        else
            console.log(message, ...optionalParams);
    }
    error(subsystem, message, ...optionalParams) {
        if (typeof message === 'string')
            message = clc_1.clc.error(message);
        if (subsystem)
            console.error(clc_1.clc.error(`[${subsystem}]`), message, ...optionalParams);
        else
            console.error(message, ...optionalParams);
    }
    info(subsystem, message, ...optionalParams) {
        if (!this.logLevelCheck(subsystem, 'info'))
            return;
        if (typeof message === 'string')
            message = clc_1.clc.info(message);
        if (subsystem)
            console.info(clc_1.clc.info(`[${subsystem}]`), message, ...optionalParams);
        else
            console.info(message, ...optionalParams);
    }
    warn(subsystem, message, ...optionalParams) {
        if (!this.logLevelCheck(subsystem, 'warn'))
            return;
        if (typeof message === 'string')
            message = clc_1.clc.warn(message);
        if (subsystem)
            console.warn(clc_1.clc.warn(`[${subsystem}]`), message, ...optionalParams);
        else
            console.warn(message, ...optionalParams);
    }
    verbose(subsystem, message, ...optionalParams) {
        if (!this.logLevelCheck(subsystem, 'verbose'))
            return;
        if (subsystem)
            console.log(`[${subsystem}]`, 'verbose:', message, ...optionalParams);
        else
            console.log('verbose:', message, ...optionalParams);
    }
    trace(subsystem, message, ...optionalParams) {
        if (!this.logLevelCheck(subsystem, 'trace'))
            return;
        if (subsystem)
            console.trace(`[${subsystem}]`, message, ...optionalParams);
        else
            console.trace(message, ...optionalParams);
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map